part of tecfy_core_client_package;

class EntityListController extends BaseController{
  
  EntityService _service;
  Entity entity;
  List<dynamic> data;
  List<EntityFilter> filters;
  int pageIndex =0;
  int pageIndexMax =0;
  bool hasNextPage =true;
  ScrollController scrollController;
  RefreshController refreshController = RefreshController(initialRefresh: false); // for pull to refrsh

  AnimationController controller;
  AnimationController counterAnimationController;
  Animation<double> sizeAnimation;
  static const double size = 800.0;
  double size1 = 0;
  double size2 = 0;
  double size3 = 0;
  double size4 = 0;

  EntityListController(State<StatefulWidget> state,String entityName) : super(state){

    refreshController.footerMode.addListener(footerListner);

    // scrollController = ScrollController();
    // scrollController.addListener(_scrollListner);

    // if(LayoutService.disableAnimation(context)){
      controller = AnimationController(
          vsync: state as TickerProviderStateMixin,
          duration: Duration(milliseconds: 600));
      sizeAnimation = Tween<double>(begin: 1.0, end: 0.0).animate(
          CurvedAnimation(parent: controller, curve: Curves.fastOutSlowIn));
      controller.addListener(() {
        size1 = size * getValueForAnimcation(sizeAnimation.value, 0.3);
        size2 = size * 2 * getValueForAnimcation(sizeAnimation.value, 0.20);
        size3 = size * 4 * getValueForAnimcation(sizeAnimation.value, 0.10);
        size4 = size * 8 * sizeAnimation.value;
        setState();
      });
    // }
    _service = EntityService();
    filters = List<EntityFilter>();
    print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName);
    loadData();
  }


  @override dispose(){
    //scrollController?.removeListener(_scrollListner);
    scrollController?.dispose();
    controller?.dispose();
    counterAnimationController?.dispose();
    refreshController?.dispose();
    super.dispose();
  }

  
 

  getValueForAnimcation(double value, double th) => (value < th) ? 0: value;
  setCountdownTimerController(){
    if(controller == null) return;
    counterAnimationController = AnimationController(vsync: state as TickerProviderStateMixin, duration: Duration(seconds: 60));
    counterAnimationController.forward(); //from: counterAnimationController.value == 0.0 ? 1.0: counterAnimationController.value
  }
  refreshAnimation() {
    if(controller == null) return;
    size1 = size;
    size2 = size * 2;
    size3 = size * 3;
    size4 = size * 4;
    controller.reset();
    controller.forward();
  }


  String sortProp;
  bool sortAsc = true;

  sortBy(Property prop){
    if(prop.dbName != sortProp){
      sortProp = prop.dbName;
      sortAsc =true;
    }else sortAsc = !sortAsc;

    data.sort((a,b)=> sortAsc ? a[sortProp].toString().compareTo(b[sortProp].toString()) : b[sortProp].toString().compareTo(a[sortProp].toString()) );
    setState();
  }

  bool get hasPageing => (data?.length ?? 0) >= EntityService.pageSize;

  footerListner()async{

    print('==========on load ====== ${refreshController.footerStatus}');
    if(refreshController.footerStatus != LoadStatus.loading) return;
    var success = await gotoMextPageIndex();
    if(success && hasNextPage)
    {
      refreshController.loadComplete();
    } 
    else if(success && !hasNextPage)
    {
      //refreshController.loadComplete();
      refreshController.loadNoData();
    }
    else{
      refreshController.loadFailed();
    }
  }

  Future<bool> gotoMextPageIndex(){
    pageIndex++;
    pageIndexMax = max(pageIndexMax,pageIndex);
    return loadData(loadFromServer:true,withLoading:false);
  }

  gotoPageIndex(int i){
    pageIndex=i;
    setLoading();
    loadData(loadFromServer:true);
  }

  var isLoading =false;
  Future<bool> loadData({bool loadFromServer=false,bool withLoading=true})async{
    if(isLoading)return true;
    isLoading=true;
    var result = await _service.getData(entity,filter:filters,forceFromServer: loadFromServer,pageIndex: pageIndex);
    if(withLoading)setResume();
    if(!result.success){
      data = [];
      if(withLoading){
        setState();
        result.showMessage(context);
      }
      isLoading=false;
      return false;
    }else{
      if(LayoutService.isMobile(context) && pageIndex > 0)
        data.addAll(result.data);
      else
        data = result.data;
      
      if(result.data.length < EntityService.pageSize)
        hasNextPage =false;
    }
    setState();
    if(pageIndex == 0) refreshAnimation();
    isLoading=false;
    return true;
  }

  Future loadFromServer()async{
    await loadData(loadFromServer:true);
    refreshController.refreshCompleted();
  }

  Future onRefresh()async{
    pageIndex =0;
    pageIndexMax =0;
    hasNextPage =true;
    setLoading();
    await loadData(loadFromServer:true);
  }

  Future onAdd()async{
    var result = await AppNavigator.navigateTo(context, ()=>EntityFormPage(entity.dbName), animationType: AnimationTypes.slideUp);
    if(result == true){
      loadFromServer();
    }
  }

  Future onUpdate(d)async {
    var result = await AppNavigator.navigateTo(context, ()=>EntityFormPage(entity.dbName,d), animationType: AnimationTypes.slideUp);
    if(result == true){
      loadFromServer();
    }
  }

  Future onDelete(d)async{
    
    if(await confirmMessage('هل انت متاكد من الحذف؟')){
      var result = await EntityService().delete(entity,d['id']);
      if(result.success){
        showMessage('تم الحذف بنجاح', type: MessageType.info);
        loadFromServer();
      }else{
        showMessage('خطأ اثناء الحذف',type: MessageType.error);
      }
    }

  }

  lowerFirst(String s){
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName => '${entity.properties.firstWhere((d)=>d.isImage).dbName}Id';
  
}